@@ -20,6 +20,7 @@
 
 #include "pyi_global.h"
 #include <stdio.h>  /* FILE */
+#include <inttypes.h>  /* uint64_t */
 
 /* Types of CArchive items. */
 #define ARCHIVE_ITEM_BINARY           'b'  /* binary */
@@ -31,13 +32,14 @@
 #define ARCHIVE_ITEM_PYSOURCE         's'  /* Python script (v3) */
 #define ARCHIVE_ITEM_DATA             'x'  /* data */
 #define ARCHIVE_ITEM_RUNTIME_OPTION   'o'  /* runtime option */
+#define ARCHIVE_ITEM_SPLASH           'l'  /* splash resources */
 
 /* TOC entry for a CArchive */
 typedef struct _toc {
     int  structlen;  /*len of this one - including full len of name */
-    int  pos;        /* pos rel to start of concatenation */
-    int  len;        /* len of the data (compressed) */
-    int  ulen;       /* len of data (uncompressed) */
+    uint32_t pos;    /* pos rel to start of concatenation */
+    uint32_t len;    /* len of the data (compressed) */
+    uint32_t ulen;   /* len of data (uncompressed) */
     char cflag;      /* is it compressed (really a byte) */
     char typcd;      /* type code -'b' binary, 'z' zlib, 'm' module,
                       * 's' script (v3),'x' data, 'o' runtime option  */
@@ -48,8 +50,8 @@ typedef struct _toc {
 /* The CArchive Cookie, from end of the archive. */
 typedef struct _cookie {
     char magic[8];      /* 'MEI\014\013\012\013\016' */
-    int  len;           /* len of entire package */
-    int  TOC;           /* pos (rel to start) of TableOfContents */
+    uint32_t len;       /* len of entire package */
+    uint32_t TOC;       /* pos (rel to start) of TableOfContents */
     int  TOClen;        /* length of TableOfContents */
     int  pyvers;        /* new in v4 */
     char pylibname[64]; /* Filename of Python dynamic library e.g. python2.7.dll. */
@@ -57,7 +59,7 @@ typedef struct _cookie {
 
 typedef struct _archive_status {
     FILE * fp;
-    size_t pkgstart;
+    uint64_t pkgstart;
     TOC *  tocbuff;
     TOC *  tocend;
     COOKIE cookie;
@@ -73,6 +75,7 @@ typedef struct _archive_status {
      *    (formerly called _Py_char2wchar) first.
      */
     char archivename[PATH_MAX];
+    char executablename[PATH_MAX];
     char homepath[PATH_MAX];
     char temppath[PATH_MAX];
     /*
@@ -129,15 +132,17 @@ void pyi_arch_status_free(ARCHIVE_STATUS *status);
 /*
  * Setup the paths and open the archive
  *
- * @param archivePath  The path including filename to the archive.
+ * @param archive_path  The path including filename to the archive (can be different from executable path).
+ * @param executable_path  The path including filename to the executable.
  *
  * @return true on success, false otherwise.
  */
-bool pyi_arch_setup(ARCHIVE_STATUS *status, char const * archivePath);
+bool pyi_arch_setup(ARCHIVE_STATUS *status, char const * archive_path, char const * executable_path);
 
 TOC *getFirstTocEntry(ARCHIVE_STATUS *status);
 TOC *getNextTocEntry(ARCHIVE_STATUS *status, TOC *entry);
 
 char * pyi_arch_get_option(const ARCHIVE_STATUS * status, char * optname);
+TOC *pyi_arch_find_by_name(ARCHIVE_STATUS *status, const char *name);
 
 #endif  /* PYI_ARCHIVE_H */
